@@ -57,7 +57,8 @@ iframe_url_requested (GtkHTML *html, const char *url, GtkHTMLStream *handle, gpo
 	HTMLIFrame *iframe = HTML_IFRAME (data);
 	GtkHTML *parent = GTK_HTML (HTML_EMBEDDED(iframe)->parent);
 
-	g_signal_emit_by_name (parent->engine, "url_requested", url, handle);
+	if (!html->engine->stopped)
+		g_signal_emit_by_name (parent->engine, "url_requested", url, handle);
 }
 
 static void
@@ -141,7 +142,7 @@ set_max_width (HTMLObject *o, HTMLPainter *painter, gint max_width)
 	HTMLEngine *e = GTK_HTML (HTML_IFRAME (o)->html)->engine;
 
 	o->max_width = max_width;
-	html_object_set_max_width (e->clue, e->painter, max_width - e->leftBorder - e->rightBorder);
+	html_object_set_max_width (e->clue, e->painter, max_width - (html_engine_get_left_border (e) + html_engine_get_right_border (e)));
 }
 
 static void
@@ -154,15 +155,6 @@ reset (HTMLObject *o)
 	html_object_reset (GTK_HTML (iframe->html)->engine->clue);
 }
 
-static void
-draw_background (HTMLObject *self,
-		 HTMLPainter *p,
-		 gint x, gint y,
-		 gint width, gint height,
-		 gint tx, gint ty)
-{
-}
-
 /* FIXME rodo - draw + set_painter is not much clean now, needs refactoring */
 
 static void
@@ -184,9 +176,9 @@ draw (HTMLObject *o,
 
 		html_object_draw (e->clue, e->painter,
 				  x, y,
-				  width - pixel_size * (e->leftBorder + e->rightBorder),
-				  height - pixel_size * (e->topBorder + e->bottomBorder),
-				  tx + pixel_size * e->leftBorder, ty + pixel_size * e->topBorder);
+				  width - pixel_size * (html_engine_get_left_border (e) + html_engine_get_right_border (e)),
+				  height - pixel_size * (html_engine_get_top_border (e) + html_engine_get_bottom_border (e)),
+				  tx + pixel_size * html_engine_get_left_border (e), ty + pixel_size * html_engine_get_top_border (e));
 	} else
 		(*HTML_OBJECT_CLASS (parent_class)->draw) (o, p, x, y, width, height, tx, ty);
 }
@@ -224,8 +216,8 @@ check_page_split (HTMLObject *self, gint y)
 	gint y1, y2;
 	HTMLEngine *e = GTK_HTML (HTML_IFRAME (self)->html)->engine;
 
-	y1 = self->y - self->ascent + e->topBorder;
-	y2 = self->y + self->descent + e->topBorder;
+	y1 = self->y - self->ascent + html_engine_get_top_border (e);
+	y2 = self->y + self->descent + html_engine_get_top_border (e);
 	
 	if (y1 > y) 
 		return 0;
@@ -568,7 +560,6 @@ html_iframe_init (HTMLIFrame *iframe,
 	GtkWidget *new_widget;
 	GtkHTML   *new_html;
 	GtkHTML   *parent_html;
-	GtkHTMLStream *handle;
 	GtkWidget *scrolled_window;
 	gint depth;
 
@@ -592,6 +583,7 @@ html_iframe_init (HTMLIFrame *iframe,
 
 	new_widget = gtk_html_new ();
 	new_html = GTK_HTML (new_widget);
+	new_html->engine->cursor_hide_count = 0;
 
 	new_tokenizer = html_tokenizer_clone (parent_html->engine->ht);
 
@@ -612,12 +604,21 @@ html_iframe_init (HTMLIFrame *iframe,
 	iframe->gdk_painter = NULL;
 	iframe->frameborder = border;
 	gtk_html_set_base (new_html, src);
+	g_signal_connect (new_html, "url_requested", G_CALLBACK (iframe_url_requested), iframe);
 
-	handle = gtk_html_begin (new_html);
+	if (depth < 10) {
+		if (parent_html->engine->stopped) {
+			gtk_html_stop (new_html);
+			gtk_html_load_empty (new_html);
+		} else {
+			GtkHTMLStream *handle;
 
+			handle = gtk_html_begin (new_html);
+			g_signal_emit_by_name (parent_html->engine, "url_requested", src, handle);
+		}
+	}
 	new_html->engine->clue->parent = HTML_OBJECT (iframe);
 	
-	g_signal_connect (new_html, "url_requested", G_CALLBACK (iframe_url_requested), iframe);
 #if 0
 	/* NOTE: because of peculiarities of the frame/gtkhtml relationship
 	 * on_url and link_clicked are emitted from the toplevel widget not
@@ -638,8 +639,6 @@ html_iframe_init (HTMLIFrame *iframe,
 	  gtk_signal_connect (GTK_OBJECT (html), "button_press_event",
 	  GTK_SIGNAL_FUNC (iframe_button_press_event), iframe);
 	*/
-	if (depth < 10)
-		g_signal_emit_by_name (parent_html->engine, "url_requested", src, handle);
 	
 	gtk_widget_set_size_request (scrolled_window, width, height);
 
@@ -708,7 +707,6 @@ html_iframe_class_init (HTMLIFrameClass *klass,
 	object_class->get_engine              = get_engine;
 	object_class->check_point             = check_point;
 	object_class->is_container            = is_container;
-	object_class->draw_background         = draw_background;
 	object_class->append_selection_string = append_selection_string;
 	
 	embedded_class->reparent = reparent;
