@@ -64,7 +64,9 @@ destroy (HTMLObject *self)
 	self->next = NULL;
 	self->prev = NULL;
 #endif
+
 	g_datalist_clear (&self->object_data);
+	g_datalist_clear (&self->object_data_nocp);
 	
 	if (self->redraw_pending) {
 		self->free_pending = TRUE;
@@ -98,6 +100,8 @@ copy (HTMLObject *self,
 
 	g_datalist_init (&dest->object_data);
 	html_object_copy_data_from_object (dest, self);
+
+	g_datalist_init (&dest->object_data_nocp);
 }
 
 static HTMLObject *
@@ -191,27 +195,6 @@ draw (HTMLObject *o,
 	/* Do nothing by default.  We don't know how to paint ourselves.  */
 }
 
-static void
-draw_background (HTMLObject *self,
-		 HTMLPainter *p,
-		 gint x, gint y,
-		 gint width, gint height,
-		 gint tx, gint ty)
-{
-	/* By default, objects are transparent so they simply forward
-           this to the parent.  */
-	if (self->parent != NULL) {
-		html_object_draw_background (self->parent, p,
-					     x + self->parent->x,
-					     y + self->parent->y - self->parent->ascent,
-					     width, height,
-					     tx - self->parent->x,
-					     ty - self->parent->y + self->parent->ascent);
-	} else {
-		/* FIXME this should draw the default background somehow.  */
-	}
-}
-
 static gboolean
 is_transparent (HTMLObject *self)
 {
@@ -226,7 +209,6 @@ fit_line (HTMLObject *o,
 	  gboolean next_to_floating,
 	  gint width_left)
 {
-	html_object_calc_size (o, painter, FALSE);
 	return (o->width <= width_left || (first_run && !next_to_floating)) ? HTML_FIT_COMPLETE : HTML_FIT_NONE;
 }
 
@@ -287,13 +269,13 @@ reset (HTMLObject *o)
 }
 
 static const gchar *
-get_url (HTMLObject *o)
+get_url (HTMLObject *o, gint offset)
 {
 	return NULL;
 }
 
 static const gchar *
-get_target (HTMLObject *o)
+get_target (HTMLObject *o, gint offset)
 {
 	return NULL;
 }
@@ -473,7 +455,7 @@ get_cursor_base (HTMLObject *self,
 static guint
 get_length (HTMLObject *self)
 {
-	return 1;
+	return html_object_accepts_cursor (self) ? 1 : 0;
 }
 
 static guint
@@ -498,7 +480,7 @@ select_range (HTMLObject *self,
 	gboolean selected;
 	gboolean changed;
 
-	selected = length > 0 || (length == -1 && start < html_object_get_length (self)) ? TRUE : FALSE;
+	selected = length > 0 || (length == -1 && start < html_object_get_length (self)) || html_object_is_container (self) ? TRUE : FALSE;
 	changed  = (! selected && self->selected) || (selected && ! self->selected) ? TRUE : FALSE;
 
 	self->selected = selected;
@@ -621,7 +603,6 @@ html_object_class_init (HTMLObjectClass *klass,
 	klass->remove_child = remove_child;
 	klass->split = split;
 	klass->draw = draw;
-	klass->draw_background = draw_background;
 	klass->is_transparent = is_transparent;
 	klass->fit_line = fit_line;
 	klass->calc_size = calc_size;
@@ -696,8 +677,10 @@ html_object_init (HTMLObject *o,
 	o->redraw_pending = FALSE;
 	o->free_pending = FALSE;
 	o->selected = FALSE;
+	o->draw_focused = FALSE;
 
 	g_datalist_init (&o->object_data);
+	g_datalist_init (&o->object_data_nocp);
 }
 
 HTMLObject *
@@ -867,16 +850,6 @@ html_object_draw (HTMLObject *o,
 	(* HO_CLASS (o)->draw) (o, p, x, y, width, height, tx, ty);
 }
 
-void
-html_object_draw_background (HTMLObject *o,
-			     HTMLPainter *p,
-			     gint x, gint y,
-			     gint width, gint height,
-			     gint tx, gint ty)
-{
-	(* HO_CLASS (o)->draw_background) (o, p, x, y, width, height, tx, ty);
-}
-
 gboolean
 html_object_is_transparent (HTMLObject *self)
 {
@@ -980,24 +953,24 @@ html_object_get_uris (HTMLObject *o, char **link, char **target, char **src)
 #endif 
 
 const gchar *
-html_object_get_url (HTMLObject *o)
+html_object_get_url (HTMLObject *o, gint offset)
 {
-	return (* HO_CLASS (o)->get_url) (o);
+	return (* HO_CLASS (o)->get_url) (o, offset);
 }
 
 const gchar *
-html_object_get_target (HTMLObject *o)
+html_object_get_target (HTMLObject *o, gint offset)
 {
-	return (* HO_CLASS (o)->get_target) (o);
+	return (* HO_CLASS (o)->get_target) (o, offset);
 }
 
 gchar *
-html_object_get_complete_url (HTMLObject *o)
+html_object_get_complete_url (HTMLObject *o, gint offset)
 {
 	const gchar *url, *target;
 
-	url = html_object_get_url (o);
-	target = html_object_get_target (o);
+	url = html_object_get_url (o, offset);
+	target = html_object_get_target (o, offset);
 	return url || target ? g_strconcat (url ? url : "#", url ? (target && *target ? "#" : NULL) : target,
 					      url ? target : NULL, NULL) : NULL;
 }
@@ -1604,6 +1577,24 @@ html_object_get_index (HTMLObject *self, guint offset)
 	return html_object_is_text (self) ? html_text_get_index (HTML_TEXT (self), offset) : offset;
 }
 
+void
+html_object_set_data_nocp (HTMLObject *object, const gchar *key, const gchar *value)
+{
+	g_datalist_set_data_full (&object->object_data_nocp, key, g_strdup (value), g_free);
+}
+
+void
+html_object_set_data_full_nocp (HTMLObject *object, const gchar *key, const gpointer value, GDestroyNotify func)
+{
+	g_datalist_set_data_full (&object->object_data_nocp, key, value, func);
+}
+
+gpointer
+html_object_get_data_nocp (HTMLObject *object, const gchar *key)
+{
+	return g_datalist_get_data (&object->object_data_nocp, key);
+}
+
 void
 html_object_set_data (HTMLObject *object, const gchar *key, const gchar *value)
 {
@@ -1648,7 +1639,7 @@ object_save_data (GQuark key_id, gpointer data, gpointer user_data)
 	/* printf ("object %s %s\n", g_quark_to_string (key_id), str); */
 	if (!str) {
 		/* printf ("save %s %s -> %s\n", state->save_data_class_name, g_quark_to_string (key_id), (gchar *) data); */
-		html_engine_save_output_string (state, "<!--+GtkHTML:<DATA class=\"%s\" key=\"%s\" value=\"%s\">",
+		html_engine_save_output_string (state, "<!--+GtkHTML:<DATA class=\"%s\" key=\"%s\" value=\"%s\">-->",
 						state->save_data_class_name, g_quark_to_string (key_id), data);
 		html_engine_set_class_data (state->engine, state->save_data_class_name, g_quark_to_string (key_id), data);
 	}
@@ -1664,12 +1655,12 @@ handle_object_data (gpointer key, gpointer value, gpointer data)
 	/* printf ("handle: %s %s %s %s\n", state->save_data_class_name, key, value, str); */
 	if (!str) {
 		/* printf ("clear\n"); */
-		html_engine_save_output_string (state, "<!--+GtkHTML:<DATA class=\"%s\" clear=\"%s\">",
+		html_engine_save_output_string (state, "<!--+GtkHTML:<DATA class=\"%s\" clear=\"%s\">-->",
 						state->save_data_class_name, key);
 		state->data_to_remove = g_slist_prepend (state->data_to_remove, key);
 	} else if (strcmp (value, str)) {
 		/* printf ("change\n"); */
-		html_engine_save_output_string (state, "<!--+GtkHTML:<DATA class=\"%s\" key=\"%s\" value=\"%s\">",
+		html_engine_save_output_string (state, "<!--+GtkHTML:<DATA class=\"%s\" key=\"%s\" value=\"%s\">-->",
 						state->save_data_class_name, key, str);
 		html_engine_set_class_data (state->engine, state->save_data_class_name, key, value);
 	}
@@ -1822,19 +1813,6 @@ html_object_get_head_leaf (HTMLObject *o)
 	return rv;
 }
 
-static void
-clear_word_width (HTMLObject *o, HTMLEngine *e, gpointer data)
-{
-	if (html_object_is_text (o))
-		html_text_clear_word_width (HTML_TEXT (o));
-}
-
-void
-html_object_clear_word_width (HTMLObject *o)
-{
-	html_object_forall (o, NULL, clear_word_width, NULL);
-}
-
 HTMLObject *
 html_object_nth_parent (HTMLObject *self, gint n)
 {
