@@ -29,6 +29,7 @@
 #include "htmlengine-edit.h"
 #include "htmlengine-save.h"
 #include "htmlpainter.h"
+#include "htmlplainpainter.h"
 #include "htmltable.h"
 #include "htmltablepriv.h"
 #include "htmltablecell.h"
@@ -44,7 +45,7 @@ static HTMLClueVClass *parent_class = NULL;
 static void
 draw_background_helper (HTMLTableCell *cell,
 			HTMLPainter *p,
-			ArtIRect *paint,
+			GdkRectangle *paint,
 			gint tx, gint ty)
 {
 	HTMLObject *o;
@@ -61,27 +62,23 @@ draw_background_helper (HTMLTableCell *cell,
 			cell->bg_allocated = TRUE;
 		}
 		color = &cell->bg;
-	} else if (t && t->bgColor) {
-		html_painter_alloc_color (p, t->bgColor);
-		color = t->bgColor;
 	}
 
 	if (cell->have_bgPixmap) {
-		if (cell->bgPixmap->pixbuf) {
-			pixbuf = cell->bgPixmap->pixbuf;
+		if (cell->bgPixmap->animation) {
+			pixbuf = gdk_pixbuf_animation_get_static_image (cell->bgPixmap->animation);
 		}
-	} else if (t && t->bgPixmap)
-		pixbuf = t->bgPixmap->pixbuf;
-
-	html_painter_draw_background (p,
-				      color,
-				      pixbuf,
-				      tx + paint->x0,
-				      ty + paint->y0,
-				      paint->x1 - paint->x0,
-				      paint->y1 - paint->y0,
-				      paint->x0 - o->x,
-				      paint->y0 - (o->y - o->ascent));
+	}
+	if (!HTML_IS_PLAIN_PAINTER (p))
+		html_painter_draw_background (p,
+					      color,
+					      pixbuf,
+					      tx + paint->x,
+					      ty + paint->y,
+					      paint->width,
+					      paint->height,
+					      paint->x - o->x,
+					      paint->y - (o->y - o->ascent));
 }
 
 
@@ -163,12 +160,11 @@ draw (HTMLObject *o,
       gint tx, gint ty)
 {
 	HTMLTableCell *cell = HTML_TABLE_CELL (o);
-	ArtIRect paint;
+	GdkRectangle paint;
 
 	
-	html_object_calc_intersection (o, &paint, x, y, width, height);
-	if (art_irect_empty (&paint))
-	    return;
+	if (!html_object_intersect (o, &paint, x, y, width, height))
+		return;
 	
 	draw_background_helper (cell, p, &paint, tx, ty);
 
@@ -226,24 +222,6 @@ calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
 	return rv;
 }
 
-static void
-draw_background (HTMLObject *self,
-		 HTMLPainter *painter,
-		 gint x, gint y, 
-		 gint width, gint height,
-		 gint tx, gint ty)
-{
-	ArtIRect paint;
-	
-	(* HTML_OBJECT_CLASS (parent_class)->draw_background) (self, painter, x, y, width, height, tx, ty);
-
-	html_object_calc_intersection (self, &paint, x, y, width, height);
-	if (art_irect_empty (&paint))
-	    return;
-
-	draw_background_helper (HTML_TABLE_CELL (self), painter, &paint, tx, ty);
-}
-
 static void
 set_bg_color (HTMLObject *object, GdkColor *color)
 {
@@ -285,7 +263,8 @@ save (HTMLObject *self,
 
 	SB cell->heading ? "<TH" : "<TD" SE;
 	if (cell->have_bg
-	    && (!HTML_TABLE (self->parent)->bgColor || !gdk_color_equal (&cell->bg, HTML_TABLE (self->parent)->bgColor)))
+	    && (!self->parent || !HTML_TABLE (self->parent)->bgColor
+		|| !gdk_color_equal (&cell->bg, HTML_TABLE (self->parent)->bgColor)))
 		SB " BGCOLOR=\"#%02x%02x%02x\"",
 			cell->bg.red >> 8,
 			cell->bg.green >> 8,
@@ -327,7 +306,7 @@ save (HTMLObject *self,
 	    && HTML_IS_CLUEFLOW (HTML_CLUE (self)->head) && html_clueflow_is_empty (HTML_CLUEFLOW (HTML_CLUE (self)->head)))
 	    SB "&nbsp;" SE; */
 
-	SB "</TD>\n" SE;
+	SB cell->heading ? "</TH>\n" : "</TD>\n" SE;
 
 	return TRUE;
 }
@@ -358,7 +337,6 @@ html_table_cell_class_init (HTMLTableCellClass *klass,
 	object_class->calc_preferred_width = calc_preferred_width;
 	object_class->calc_size = calc_size;
 	object_class->draw = draw;
-	object_class->draw_background = draw_background;
 	object_class->set_bg_color = set_bg_color;
 	object_class->get_bg_color = get_bg_color;
 	object_class->save = save;
@@ -458,3 +436,14 @@ html_table_cell_get_fixed_width (HTMLTableCell *cell, HTMLPainter *painter)
 {
 	return html_painter_get_pixel_size (painter) * cell->fixed_width;
 }
+
+gboolean
+html_table_cell_is_empty (HTMLTableCell *cell)
+{
+	g_assert (HTML_IS_TABLE_CELL (cell));
+
+	if (HTML_CLUE (cell)->head && HTML_CLUE (cell)->head == HTML_CLUE (cell)->tail
+	    && HTML_IS_CLUEFLOW (HTML_CLUE (cell)->head) && html_clueflow_is_empty (HTML_CLUEFLOW (HTML_CLUE (cell)->head)))
+		return TRUE;
+	return FALSE;
+}
