@@ -16,14 +16,30 @@
     Boston, MA 02111-1307, USA.
 */
 
-#define MEMDEBUG
+#include "config.h"
+#include <libgnome/gnome-i18n.h>
+#include <gnome.h>
+#include <bonobo.h>
+#include <sys/types.h>
 
+#include <glib.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+ 
 #include <gnome.h>
 #include <gtk/gtk.h>
+#include <gtk/gtkbutton.h>
 
 #include <libgnomeprint/gnome-print.h>
-#include <libgnomeprint/gnome-print-master.h>
-#include <libgnomeprint/gnome-print-master-preview.h>
+#include <libgnomeprint/gnome-print-job.h>
+#include <libgnomeprintui/gnome-print-job-preview.h>
+
+#include <libsoup/soup.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <errno.h>
+#include <unistd.h>
+#include <string.h>
 
 #include "config.h"
 #include "gtkhtml.h"
@@ -34,20 +50,6 @@
 
 #include "gtkhtmldebug.h"
 
-#undef PACKAGE
-#undef VERSION
-
-#include <WWWApp.h>
-#include <WWWLib.h>
-#include <WWWStream.h>
-#include <WWWInit.h>
-#undef PACKAGE
-#undef VERSION
-
-#include "config.h"
-
-#include <glibwww/glibwww.h>
-
 typedef struct {
   FILE *fil;
   GtkHTMLStream *handle;
@@ -66,6 +68,7 @@ static void print_preview_cb (GtkWidget *widget, gpointer data);
 static void test_cb (GtkWidget *widget, gpointer data);
 static void bug_cb (GtkWidget *widget, gpointer data);
 static void slow_cb (GtkWidget *widget, gpointer data);
+static void animate_cb (GtkWidget *widget, gpointer data);
 static void stop_cb (GtkWidget *widget, gpointer data);
 static void dump_cb (GtkWidget *widget, gpointer data);
 static void dump_simple_cb (GtkWidget *widget, gpointer data);
@@ -75,35 +78,38 @@ static void home_cb (GtkWidget *widget, gpointer data);
 static void reload_cb (GtkWidget *widget, gpointer data);
 static void redraw_cb (GtkWidget *widget, gpointer data);
 static void resize_cb (GtkWidget *widget, gpointer data);
+static void select_all_cb (GtkWidget *widget, gpointer data);
 static void title_changed_cb (GtkHTML *html, const gchar *title, gpointer data);
 static void url_requested (GtkHTML *html, const char *url, GtkHTMLStream *handle, gpointer data);
 static void entry_goto_url(GtkWidget *widget, gpointer data);
 static void goto_url(const char *url, int back_or_forward);
 static void on_set_base (GtkHTML *html, const gchar *url, gpointer data);
 
-static int netin_stream_put_character (HTStream * me, char c);
-static int netin_stream_put_string (HTStream * me, const char * s);;
-static int netin_stream_write (HTStream * me, const char * s, int l);
-static int netin_stream_flush (HTStream * me);
-static int netin_stream_free (HTStream * me);
-static int netin_stream_abort (HTStream * me, HTList * e);
-static HTStream *netin_stream_new (GtkHTMLStream *handle, HTRequest *request);
-static int redirectFilter(HTRequest *request, HTResponse *response, void *param, int status);
 static gchar *parse_href (const gchar *s);
 
+#ifdef HAVE_OLD_SOUP
+#define SOUP_STATUS_IS_SUCCESSFUL SOUP_ERROR_IS_SUCCESSFUL
+#define status_code errorcode
+#define reason_phrase errorphrase
+#else
+static SoupSession *session;
+#endif
+
 static GtkHTML *html;
 static GtkHTMLStream *html_stream_handle = NULL;
-static GtkWidget *animator, *entry;
+/* static GtkWidget *animator; */
+static GtkWidget *entry;
 static GtkWidget *popup_menu, *popup_menu_back, *popup_menu_forward, *popup_menu_home;
 static GtkWidget *toolbar_back, *toolbar_forward;
-static HTMLURL *baseURL;
+static HTMLURL *baseURL = NULL;
 
 static GList *go_list;
 static int go_position;
 
 static gboolean use_redirect_filter;
 
-static gboolean slow_loading = FALSE, exit_when_done = FALSE;
+static gboolean slow_loading = FALSE;
+static gboolean exit_when_done = FALSE;
 
 static gint redirect_timerId = 0;
 static gchar *redirect_url = NULL;
@@ -154,6 +160,10 @@ static GnomeUIInfo debug_menu[] = {
 	  resize_cb, NULL, NULL, 0, 0, 0 },
 	{ GNOME_APP_UI_ITEM, "Force repaint", "Force a repaint event",
 	  redraw_cb, NULL, NULL, 0, 0, 0 },
+	{ GNOME_APP_UI_ITEM, "Select all", "Select all",
+	  select_all_cb, NULL, NULL, 0, 0, 0 },
+	GNOMEUIINFO_TOGGLEITEM ("Disable Animations", "Disable Animated Images",  animate_cb, NULL),
+	 
 	GNOMEUIINFO_END
 };
 
@@ -180,64 +190,58 @@ static void
 create_toolbars (GtkWidget *app)
 {
 	GtkWidget *dock;
-	GtkWidget *hbox, *hbox2;
+	GtkWidget *hbox;
 	GtkWidget *frame;
-	GtkWidget *button;
 	GtkWidget *toolbar;
-	char *imgloc;
+	/* char *imgloc; */
 
-	dock = gnome_dock_item_new ("testgtkhtml-toolbar1",
-				    (GNOME_DOCK_ITEM_BEH_EXCLUSIVE));
+	dock = bonobo_dock_item_new ("testgtkhtml-toolbar1",
+				    (BONOBO_DOCK_ITEM_BEH_EXCLUSIVE));
 	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (dock), hbox);
-	gtk_container_border_width (GTK_CONTAINER (dock), 2);
+	gtk_container_set_border_width (GTK_CONTAINER (dock), 2);
 	
-	toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL,
-				   GTK_TOOLBAR_ICONS);
-	gtk_toolbar_set_style (GTK_TOOLBAR (toolbar),
-			       GTK_TOOLBAR_ICONS);
-	gtk_toolbar_set_space_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_SPACE_LINE);
-	gtk_toolbar_set_button_relief (GTK_TOOLBAR (toolbar),
-				       GTK_RELIEF_NONE);
+	toolbar = gtk_toolbar_new ();
+	gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_ICONS);
 	gtk_box_pack_start (GTK_BOX (hbox), toolbar, FALSE, FALSE, 0);
 
 	toolbar_back = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
-				 NULL,
-				 "Move back",
-				 "Back",
-				 gnome_stock_new_with_icon (GNOME_STOCK_PIXMAP_BACK),
-				 back_cb, NULL);
+						NULL,
+						"Move back",
+						"Back",
+						gtk_image_new_from_stock ("gtk-go-back", GTK_ICON_SIZE_LARGE_TOOLBAR),
+						GTK_SIGNAL_FUNC (back_cb), NULL);
 	gtk_widget_set_sensitive(toolbar_back, FALSE);
 	toolbar_forward = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
-				 NULL,
-				 "Move forward",
-				 "Forward",
-				 gnome_stock_new_with_icon (GNOME_STOCK_PIXMAP_FORWARD),
-				 forward_cb, NULL);
+						   NULL,
+						   "Move forward",
+						   "Forward",
+						   gtk_image_new_from_stock ("gtk-go-forward", GTK_ICON_SIZE_LARGE_TOOLBAR),
+						   GTK_SIGNAL_FUNC (forward_cb), NULL);
 	gtk_widget_set_sensitive(toolbar_forward, FALSE);
 	gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
 	gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
 				 NULL,
 				 "Stop loading",
 				 "Stop",
-				 gnome_stock_new_with_icon (GNOME_STOCK_PIXMAP_STOP),
-				 stop_cb,
+				 gtk_image_new_from_stock ("gtk-stop", GTK_ICON_SIZE_LARGE_TOOLBAR),
+				 GTK_SIGNAL_FUNC (stop_cb),
 				 NULL);
 	gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
 				 NULL,
 				 "Reload page",
 				 "Reload",
-				 gnome_stock_new_with_icon (GNOME_STOCK_PIXMAP_REFRESH),
-				 reload_cb, NULL);
+				 gtk_image_new_from_stock ("gtk-refresh", GTK_ICON_SIZE_LARGE_TOOLBAR),
+				 GTK_SIGNAL_FUNC (reload_cb), NULL);
 	gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
 				 NULL,
 				 "Home page",
 				 "Home",
-				 gnome_stock_new_with_icon (GNOME_STOCK_PIXMAP_HOME),
-				 home_cb, NULL);
+				 gtk_image_new_from_stock ("gtk-home", GTK_ICON_SIZE_LARGE_TOOLBAR),
+				 GTK_SIGNAL_FUNC (home_cb), NULL);
 	gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
 
-	animator = gnome_animator_new_with_size (32, 32);
+	/* animator = gnome_animator_new_with_size (32, 32);
 
 	if (g_file_exists("32.png"))
 	  imgloc = "32.png";
@@ -250,57 +254,49 @@ create_toolbars (GtkWidget *app)
 							0, 0,
 							25,
 							32, 
-							32, 32);
+							32, 32); */
 
 	frame = gtk_frame_new (NULL);
-	gtk_container_add (GTK_CONTAINER (frame), animator);
+	/* TODO2 gtk_container_add (GTK_CONTAINER (frame), animator); */
 	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
 	gtk_box_pack_end (GTK_BOX (hbox), frame, FALSE, FALSE, 0);
-	gnome_animator_set_loop_type (GNOME_ANIMATOR (animator), 
-				      GNOME_ANIMATOR_LOOP_RESTART);
+	/* gnome_animator_set_loop_type (GNOME_ANIMATOR (animator), 
+	   GNOME_ANIMATOR_LOOP_RESTART); */
 	gtk_widget_show_all (dock);
-	gnome_dock_add_item (GNOME_DOCK (GNOME_APP (app)->dock),
-			     GNOME_DOCK_ITEM (dock), GNOME_DOCK_TOP, 1, 0, 0, FALSE);
+	bonobo_dock_add_item (BONOBO_DOCK (GNOME_APP (app)->dock),
+			      BONOBO_DOCK_ITEM (dock), BONOBO_DOCK_TOP, 1, 0, 0, FALSE);
 
 	/* Create the location bar */
-	dock = gnome_dock_item_new ("testgtkhtml-toolbar2",
-				    (GNOME_DOCK_ITEM_BEH_EXCLUSIVE));
+	dock = bonobo_dock_item_new ("testgtkhtml-toolbar2",
+				     (BONOBO_DOCK_ITEM_BEH_EXCLUSIVE));
 	hbox = gtk_hbox_new (FALSE, 2);
 	gtk_container_add (GTK_CONTAINER (dock), hbox);
-	gtk_container_border_width (GTK_CONTAINER (dock), 2);
-	hbox2 = gtk_hbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox2), gtk_label_new ("Goto"), FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox2), gnome_stock_new_with_icon (GNOME_STOCK_PIXMAP_REDO), FALSE, FALSE, 0);
-	button = gtk_button_new ();
-	GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
-	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
-	gtk_container_add (GTK_CONTAINER (button), hbox2);
-	gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (dock), 2);
 	gtk_box_pack_start (GTK_BOX (hbox),
 			    gtk_label_new ("Location:"), FALSE, FALSE, 0);
 	entry = gtk_entry_new ();
-	gtk_signal_connect(GTK_OBJECT(entry), "activate", GTK_SIGNAL_FUNC(entry_goto_url), NULL);
+	g_signal_connect (entry, "activate", G_CALLBACK (entry_goto_url), NULL);
 	gtk_box_pack_start (GTK_BOX (hbox),
 			    entry, TRUE, TRUE, 0);
-	gnome_dock_add_item (GNOME_DOCK (GNOME_APP (app)->dock),
-			     GNOME_DOCK_ITEM (dock), GNOME_DOCK_TOP, 2, 0, 0, FALSE);
+	bonobo_dock_add_item (BONOBO_DOCK (GNOME_APP (app)->dock),
+			      BONOBO_DOCK_ITEM (dock), BONOBO_DOCK_TOP, 2, 0, 0, FALSE);
 
 }
 
-static gint page_num;
+static gint page_num, pages;
 static GnomeFont *font;
 
 static void
 print_footer (GtkHTML *html, GnomePrintContext *context,
 	      gdouble x, gdouble y, gdouble width, gdouble height, gpointer user_data)
 {
-	gchar *text = g_strdup_printf ("- %d -", page_num);
-	gdouble tw = gnome_font_get_width_string (font, "text");
+	gchar *text = g_strdup_printf ("- %d of %d -", page_num, pages);
+	gdouble tw = gnome_font_get_width_utf8 (font, "text");
 
 	if (font) {
 		gnome_print_newpath     (context);
 		gnome_print_setrgbcolor (context, .0, .0, .0);
-		gnome_print_moveto      (context, x + (width - tw)/2, y - (height + gnome_font_get_ascender (font))/2);
+		gnome_print_moveto      (context, x + (width - tw)/2, y - gnome_font_get_ascender (font));
 		gnome_print_setfont     (context, font);
 		gnome_print_show        (context, text);
 	}
@@ -313,25 +309,31 @@ static void
 print_preview_cb (GtkWidget *widget,
 		  gpointer data)
 {
-	GnomePrintMaster *print_master;
+	GnomePrintJob *print_master;
 	GnomePrintContext *print_context;
 	GtkWidget *preview;
 
-	print_master = gnome_print_master_new ();
-	gnome_print_master_set_paper (print_master, gnome_paper_with_name ("US-Letter"));
+	print_master = gnome_print_job_new (NULL);
+	/* FIX2 gnome_print_master_set_paper (print_master, gnome_paper_with_name ("US-Letter")); */
 	gtk_html_print_set_master (html, print_master);
 
-	print_context = gnome_print_master_get_context (print_master);
+	print_context = gnome_print_job_get_context (print_master);
+	font = gnome_font_find_closest ("Helvetica", 12);
 
 	page_num = 1;
-	font = gnome_font_new_closest ("Helvetica", GNOME_FONT_BOOK, FALSE, 12);
-	gtk_html_print_with_header_footer (html, print_context, .0, .03, NULL, print_footer, NULL);
-	if (font) gtk_object_unref (GTK_OBJECT (font));
-
-	preview = GTK_WIDGET (gnome_print_master_preview_new (print_master, "HTML Print Preview"));
+	pages = gtk_html_print_get_pages_num (html, print_context,
+					      .0, gnome_font_get_ascender (font) - gnome_font_get_descender (font));
+	gtk_html_print_with_header_footer (html, print_context,
+					   .0, gnome_font_get_ascender (font) - gnome_font_get_descender (font),
+					   NULL, print_footer, NULL);
+	if (font)
+		g_object_unref (font);
+	
+	gnome_print_job_close (print_master);
+	preview = gnome_print_job_preview_new (print_master, "Print Preview");
 	gtk_widget_show (preview);
 
-	gtk_object_unref (GTK_OBJECT (print_master));
+	g_object_unref (print_master);
 }
 
 static void
@@ -359,11 +361,18 @@ resize_cb (GtkWidget *widget, gpointer data)
 	html_engine_calc_size (html->engine, FALSE);
 }
 
+static void
+select_all_cb (GtkWidget *widget, gpointer data)
+{
+	g_print ("select all\n");
+	gtk_html_select_all (html);
+}
+
 static void
 redraw_cb (GtkWidget *widget, gpointer data)
 {
 	g_print ("forcing redraw\n");
-	gtk_widget_draw (GTK_WIDGET (html), NULL);
+	gtk_widget_queue_draw (GTK_WIDGET (html));
 }
 
 static void
@@ -372,6 +381,12 @@ slow_cb (GtkWidget *widget, gpointer data)
 	slow_loading = !slow_loading;
 }
 
+static void
+animate_cb (GtkWidget *widget, gpointer data)
+{
+	/* gtk_html_set_animate (html, !gtk_html_get_animate (html)); */
+}
+
 static void
 title_changed_cb (GtkHTML *html, const gchar *title, gpointer data)
 {
@@ -390,9 +405,10 @@ entry_goto_url(GtkWidget *widget, gpointer data)
 	tmpurl = g_strdup (gtk_entry_get_text (GTK_ENTRY (widget)));
 
 	/* Add "http://" if no protocol is specified */
-	if(strchr(tmpurl, ':'))
+	if(strchr(tmpurl, ':')) {
+		on_set_base (NULL, tmpurl, NULL);
 		goto_url (tmpurl, 0);
-	else {
+	} else {
 		gchar *url;
 
 		url = g_strdup_printf("http://%s", tmpurl);
@@ -473,18 +489,22 @@ static void
 stop_cb (GtkWidget *widget, gpointer data)
 {
 	/* Kill all requests */
-	HTNet_killAll();
+#ifdef HAVE_OLD_SOUP
+	soup_shutdown ();
+#else
+	soup_session_abort (session);
+#endif
 	html_stream_handle = NULL;
 }
 
 static void
 load_done (GtkHTML *html)
 {
-	gnome_animator_stop (GNOME_ANIMATOR (animator));
+	/* TODO2 gnome_animator_stop (GNOME_ANIMATOR (animator));
 	gnome_animator_goto_frame (GNOME_ANIMATOR (animator), 1);
 
 	if (exit_when_done)
-		gtk_main_quit();
+	gtk_main_quit(); */
 }
 
 static int
@@ -519,6 +539,10 @@ on_set_base (GtkHTML *html, const gchar *url, gpointer data)
 	if (baseURL)
 		html_url_destroy (baseURL);
 
+	if (html) {
+		gtk_html_set_base (html, url);
+	} 
+		
 	baseURL = html_url_new (url);
 }
 
@@ -583,78 +607,6 @@ on_link_clicked (GtkHTML *html, const gchar *url, gpointer data)
 	goto_url (url, 0);
 }
 
-/* Lame hack */
-struct _HTStream {
-	const HTStreamClass *	isa;
-	GtkHTMLStream *handle;
-};
-
-static int
-netin_stream_put_character (HTStream * me, char c)
-{
-	return netin_stream_write(me, &c, 1);
-}
-
-static int
-netin_stream_put_string (HTStream * me, const char * s)
-{
-	return netin_stream_write(me, s, strlen(s));
-}
-
-static int
-netin_stream_write (HTStream * me, const char * s, int l)
-{
-	printf ("*** %s (%d)\n",__FUNCTION__, l);
-	gtk_html_write(html, me->handle, s, l);
-
-	return HT_OK;
-}
-
-static int
-netin_stream_flush (HTStream * me)
-{
-	return HT_OK;
-}
-
-static int
-netin_stream_free (HTStream * me)
-{
-	gtk_html_end(html, me->handle, GTK_HTML_STREAM_OK);
-	g_free(me);
-	
-	return HT_OK;
-}
-
-static int
-netin_stream_abort (HTStream * me, HTList * e)
-{
-	return HT_OK;
-}
-
-static const HTStreamClass netin_stream_class =
-{		
-	"netin_stream",
-	netin_stream_flush,
-	netin_stream_free,
-	netin_stream_abort,
-	netin_stream_put_character,
-	netin_stream_put_string,
-	netin_stream_write
-}; 
-
-static HTStream *
-netin_stream_new (GtkHTMLStream *handle, HTRequest *request)
-{
-	HTStream *retval;
-	
-	retval = g_new0(HTStream, 1);
-	
-	retval->isa = &netin_stream_class;
-	retval->handle = handle;
-
-	return retval;
-}
-
 /* simulate an async object isntantiation */
 static int
 object_timeout(GtkHTMLEmbedded *eb)
@@ -677,7 +629,7 @@ object_requested_cmd (GtkHTML *html, GtkHTMLEmbedded *eb, void *data)
 {
 	/* printf("object requested, wiaint a bit before creating it ...\n"); */
 
-	if (strcmp (eb->classid, "mine:NULL") == 0)
+	if (eb->classid && strcmp (eb->classid, "mine:NULL") == 0)
 		return FALSE;
 
 	gtk_widget_ref (GTK_WIDGET (eb));
@@ -687,41 +639,72 @@ object_requested_cmd (GtkHTML *html, GtkHTMLEmbedded *eb, void *data)
 	return TRUE;
 }
 
+static void
+got_data (SoupMessage *msg, gpointer user_data)
+{
+	GtkHTMLStream *handle = user_data;
+
+	if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
+		g_warning ("%d - %s", msg->status_code, msg->reason_phrase);
+		gtk_html_end (html, handle, GTK_HTML_STREAM_ERROR);
+		return;
+	}
+
+	gtk_html_write (html, handle, msg->response.body, msg->response.length);
+	gtk_html_end (html, handle, GTK_HTML_STREAM_OK);
+}
+
 static void
 url_requested (GtkHTML *html, const char *url, GtkHTMLStream *handle, gpointer data)
 {
 	gchar *full_url = NULL;
-	HTRequest *newreq;
-	BOOL status;
 
-	newreq = HTRequest_new();
-	g_assert(newreq);
-	
- 	/* Need to catch redirections */
- 	/* Remove old filter if any */
- 	HTNet_deleteAfter(redirectFilter);
- 	/* Add new filters */
 	full_url = parse_href (url);
 
-	if (use_redirect_filter) {
-		
-		use_redirect_filter = FALSE;
-		
-		HTNet_addAfter(redirectFilter, full_url, NULL, HT_PERM_REDIRECT, HT_FILTER_FIRST);
-		HTNet_addAfter(redirectFilter, full_url, NULL, HT_TEMP_REDIRECT, HT_FILTER_FIRST);
-		HTNet_addAfter(redirectFilter, full_url, NULL, HT_SEE_OTHER, HT_FILTER_FIRST);
-		HTNet_addAfter(redirectFilter, full_url, NULL, HT_FOUND, HT_FILTER_FIRST);
-	}
+	if (full_url && !strncmp (full_url, "http", 4)) {
+		SoupMessage *msg;
+#ifdef HAVE_OLD_SOUP
+		SoupContext *ctx;
 
-	HTRequest_setOutputFormat(newreq, WWW_SOURCE);
-	{
-		HTStream *newstream = netin_stream_new(handle, newreq);
-		status = HTLoadToStream(full_url, newstream, newreq);
-		g_message("Loading URL %s to stream %p (status %d)", full_url, handle, status);
-	}
-	if (full_url)
-		g_free (full_url);
-	return;
+		ctx = soup_context_get (full_url);
+		msg = soup_message_new (ctx, SOUP_METHOD_GET);
+
+		soup_message_queue (msg, got_data, handle);
+#else
+		msg = soup_message_new (SOUP_METHOD_GET, full_url);
+		soup_session_queue_message (session, msg, got_data, handle);
+#endif
+	} else if (full_url && !strncmp (full_url, "file:", 5)) {
+		struct stat st;
+		char *buf;
+		int fd, nread, total;
+
+		fd = open (full_url + 5, O_RDONLY);
+		if (fd != -1 && fstat (fd, &st) != -1) {
+			buf = g_malloc (st.st_size);
+			for (nread = total = 0; total < st.st_size; total += nread) {
+				nread = read (fd, buf + total, st.st_size - total);
+				if (nread == -1) {
+					if (errno == EINTR)
+						continue;
+
+					g_warning ("read error: %s", g_strerror (errno));
+					gtk_html_end (html, handle, GTK_HTML_STREAM_ERROR);
+					break;
+				}
+				gtk_html_write (html, handle, buf + total, nread);
+			}
+			g_free (buf);
+			if (nread != -1)
+				gtk_html_end (html, handle, GTK_HTML_STREAM_OK);
+		} else
+			gtk_html_end (html, handle, GTK_HTML_STREAM_OK);
+		if (fd != -1)
+			close (fd);
+	} else
+		g_warning ("Unrecognized URL %s", full_url);
+
+	g_free (full_url);
 }
 
 static gchar *
@@ -732,7 +715,7 @@ parse_href (const gchar *s)
 	HTMLURL *tmpurl;
 
 	if(s == NULL || *s == 0)
-		return NULL;
+		return g_strdup ("");
 
 	if (s[0] == '#') {
 		tmpurl = html_url_dup (baseURL, HTML_URL_DUP_NOREFERENCE);
@@ -839,7 +822,11 @@ goto_url(const char *url, int back_or_forward)
 	gchar *full_url;
 
 	/* Kill all requests */
-	HTNet_killAll();
+#ifdef HAVE_OLD_SOUP
+	soup_shutdown ();
+#else
+	soup_session_abort (session);
+#endif
 
 	/* Remove any pending redirection */
 	if(redirect_timerId) {
@@ -848,17 +835,18 @@ goto_url(const char *url, int back_or_forward)
 		redirect_timerId = 0;
 	}
 
-	gnome_animator_start (GNOME_ANIMATOR (animator));
+	/* TODO2 gnome_animator_start (GNOME_ANIMATOR (animator)); */
 
 	use_redirect_filter = TRUE;
 
-	html_stream_handle = gtk_html_begin (html);
+	html_stream_handle = gtk_html_begin_content (html, "text/html; charset=utf-8");
 
-	/* Yuck yuck yuck.  Well this code is butt-ugly already anyway.  */
-	url_requested (html, url, html_stream_handle, NULL);
+	/* Yuck yuck yuck.  Well this code is butt-ugly already
+	anyway.  */
 
 	full_url = parse_href (url);
 	on_set_base (NULL, full_url, NULL);
+	url_requested (html, url, html_stream_handle, NULL);
 
 	if(!back_or_forward) {
 		if(go_position) {
@@ -915,15 +903,15 @@ goto_url(const char *url, int back_or_forward)
 			item = g_list_nth_data(go_list, i);
 			item->widget = gtk_radio_menu_item_new_with_label(group, item->url);
 
-			gtk_signal_connect (GTK_OBJECT (item->widget), "activate",
-					    GTK_SIGNAL_FUNC (go_list_cb), GINT_TO_POINTER (i));
+			g_signal_connect (item->widget, "activate",
+					  G_CALLBACK (go_list_cb), GINT_TO_POINTER (i));
 
-			group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(item->widget));
+			group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(item->widget));
 
 			if(i == 0)
 				gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item->widget), TRUE);
 
-			gtk_menu_append(GTK_MENU(GTK_MENU_ITEM(main_menu[3].widget)->submenu), item->widget);
+			gtk_menu_shell_append (GTK_MENU_SHELL (GTK_MENU_ITEM(main_menu[3].widget)->submenu), item->widget);
 			gtk_widget_show(item->widget);
 			
 		}
@@ -951,19 +939,22 @@ goto_url(const char *url, int back_or_forward)
 static void
 bug_cb (GtkWidget *widget, gpointer data)
 {
-	gchar *filename;
+	gchar cwd[PATH_MAX], *filename;
 
-	filename = g_strdup_printf ("%s/tests/bugs.html", HTGetCurrentDirectoryURL());
-	goto_url (filename, 0);
-	g_free (filename);
+	getcwd(cwd, sizeof (cwd));
+	filename = g_strdup_printf("file:%s/bugs.html", cwd);
+	goto_url(filename, 0);
+	g_free(filename);
 }
 
 static void
 test_cb (GtkWidget *widget, gpointer data)
 {
-	gchar *filename;
-	
-	filename = g_strdup_printf ("%s/tests/test%d.html", HTGetCurrentDirectoryURL(), GPOINTER_TO_INT (data));
+	gchar cwd[PATH_MAX], *filename;
+
+	getcwd(cwd, sizeof (cwd));
+	filename = g_strdup_printf ("file:%s/tests/test%d.html", cwd,
+				    GPOINTER_TO_INT (data));
 	goto_url(filename, 0);
 	g_free(filename);
 }
@@ -974,56 +965,6 @@ exit_cb (GtkWidget *widget, gpointer data)
 	gtk_main_quit ();
 }
 
-static int request_terminater (HTRequest * request, HTResponse * response, void * param, int status) {
-	if (status!=HT_LOADED) 
-		g_print("Load couldn't be completed successfully (%p)\n", request);
-	
-	HTRequest_delete(request);
-	return HT_OK;
-}
-static int redirectFilter(HTRequest *request, HTResponse *response,
-		 void *param, int status)
-{
-	HTMethod method = HTRequest_method(request);
-	HTAnchor *new_anchor = HTResponse_redirection(response);
-	gchar *new_location;
-
-	if (!new_anchor)
-		return HT_OK;
-
-	if (!HTMethod_isSafe(method))
-		return HT_OK;
-
-	HTRequest_deleteCredentialsAll(request);
-	if (HTRequest_doRetry(request)) {
-		HTRequest_setAnchor(request, new_anchor);
-		HTLoad(request, NO);
-
-		new_location = HTAnchor_address(new_anchor);
-
-		g_print("******* Redirected to '%s' *****\n", new_location);
-
-		/* Update BASE url */
-
-		on_set_base (NULL, new_location, NULL);
-
-		/* Update filters */
-		HTNet_deleteAfter(redirectFilter);
-		HTNet_addAfter(redirectFilter, new_location, NULL, HT_PERM_REDIRECT, HT_FILTER_FIRST);
-		HTNet_addAfter(redirectFilter, new_location, NULL, HT_TEMP_REDIRECT, HT_FILTER_FIRST);
-		HTNet_addAfter(redirectFilter, new_location, NULL, HT_FOUND, HT_FILTER_FIRST);
-		HTNet_addAfter(redirectFilter, new_location, NULL, HT_SEE_OTHER, HT_FILTER_FIRST);
-		
-		free(new_location);
-
-	} else {
-		HTRequest_addError(request, ERR_FATAL, NO, HTERR_MAX_REDIRECT,
-				   NULL, 0, "HTRedirectFilter");
-		return HT_OK;
-	}
-	return HT_ERROR;
-}
-
 static struct poptOption options[] = {
   {"slow-loading", '\0', POPT_ARG_NONE, &slow_loading, 0, "Load the document as slowly as possible", NULL},
   {"exit-when-done", '\0', POPT_ARG_NONE, &exit_when_done, 0, "Exit the program as soon as the document is loaded", NULL},
@@ -1036,37 +977,21 @@ main (gint argc, gchar *argv[])
 	GtkWidget *app, *bar;
 	GtkWidget *html_widget;
 	GtkWidget *scrolled_window;
-	/* GdkColor   bgColor = {0, 0xc7ff, 0xc7ff, 0xc7ff}; */
-	poptContext ctx;
-#ifdef GTKHTML_HAVE_GCONF
 	GError  *gconf_error  = NULL;
-#endif
 
 #ifdef MEMDEBUG
 	void *p = malloc (1024);	/* to make linker happy with ccmalloc */
 #endif
-	gnome_init_with_popt_table (PACKAGE, VERSION,
-				    argc, argv, options, 0, &ctx);
-	glibwww_init (PACKAGE, VERSION);
-#ifdef GTKHTML_HAVE_GCONF
-	if (!gconf_init (argc, argv, &gconf_error)) {
-		g_assert (gconf_error != NULL);
-		g_error ("GConf init failed:\n  %s", gconf_error->message);
-		return FALSE;
-	}
-#endif
+	/* gnome_init_with_popt_table (PACKAGE, VERSION,
+	   argc, argv, options, 0, &ctx); */
+	gnome_program_init ("testgtkhtml", VERSION, LIBGNOMEUI_MODULE, argc, argv,
 
-	HTNet_addAfter(request_terminater, NULL, NULL, HT_ALL, HT_FILTER_LAST);
+			    GNOME_PARAM_HUMAN_READABLE_NAME, _("GtkHTML Test Application"),
+			    NULL);
 
-	gdk_rgb_init ();
-	
-	gtk_widget_set_default_colormap (gdk_rgb_get_cmap ());
-	gtk_widget_set_default_visual (gdk_rgb_get_visual ());
-	
 	app = gnome_app_new ("testgtkhtml", "GtkHTML: testbed application");
 
-	gtk_signal_connect (GTK_OBJECT (app), "delete_event",
-			    GTK_SIGNAL_FUNC (exit_cb), NULL);
+	g_signal_connect (app, "delete_event", G_CALLBACK (exit_cb), NULL);
 
 	create_toolbars (app);
 	bar = gnome_appbar_new (FALSE, TRUE, GNOME_PREFERENCES_USER);
@@ -1087,8 +1012,13 @@ main (gint argc, gchar *argv[])
 
 	gnome_app_set_contents (GNOME_APP (app), scrolled_window);
 
+#ifndef HAVE_OLD_SOUP
+	session = soup_session_async_new ();
+#endif
+
 	html_widget = gtk_html_new ();
 	html = GTK_HTML (html_widget);
+	gtk_html_set_allow_frameset (html, TRUE);
 	gtk_html_load_empty (html);
 	/* gtk_html_set_default_background_color (GTK_HTML (html_widget), &bgColor); */
 	/* gtk_html_set_editable (GTK_HTML (html_widget), TRUE); */
@@ -1098,50 +1028,37 @@ main (gint argc, gchar *argv[])
 	/* Create a popup menu with disabled back and forward items */
 	popup_menu = gtk_menu_new();
 
-	popup_menu_back = gtk_menu_item_new_with_label("Back");
-	gtk_widget_set_sensitive(popup_menu_back, FALSE);
-	gtk_menu_append(GTK_MENU(popup_menu), popup_menu_back);
-	gtk_widget_show(popup_menu_back);
-	gtk_signal_connect (GTK_OBJECT (popup_menu_back), "activate",
-			    GTK_SIGNAL_FUNC (back_cb), NULL);
+	popup_menu_back = gtk_menu_item_new_with_label ("Back");
+	gtk_widget_set_sensitive (popup_menu_back, FALSE);
+	gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), popup_menu_back);
+	gtk_widget_show (popup_menu_back);
+	g_signal_connect (popup_menu_back, "activate", G_CALLBACK (back_cb), NULL);
 
-	popup_menu_forward = gtk_menu_item_new_with_label("Forward");
-	gtk_widget_set_sensitive(popup_menu_forward, FALSE);
-	gtk_menu_append(GTK_MENU(popup_menu), popup_menu_forward);
-	gtk_widget_show(popup_menu_forward);
-	gtk_signal_connect (GTK_OBJECT (popup_menu_forward), "activate",
-			    GTK_SIGNAL_FUNC (forward_cb), NULL);
+	popup_menu_forward = gtk_menu_item_new_with_label ("Forward");
+	gtk_widget_set_sensitive (popup_menu_forward, FALSE);
+	gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), popup_menu_forward);
+	gtk_widget_show (popup_menu_forward);
+	g_signal_connect (popup_menu_forward, "activate", G_CALLBACK (forward_cb), NULL);
 
-	popup_menu_home = gtk_menu_item_new_with_label("Home");
-	gtk_menu_append(GTK_MENU(popup_menu), popup_menu_home);
-	gtk_widget_show(popup_menu_home);
-	gtk_signal_connect (GTK_OBJECT (popup_menu_home), "activate",
-			    GTK_SIGNAL_FUNC (home_cb), NULL);
+	popup_menu_home = gtk_menu_item_new_with_label ("Home");
+	gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), popup_menu_home);
+	gtk_widget_show (popup_menu_home);
+	g_signal_connect (popup_menu_home, "activate", G_CALLBACK (home_cb), NULL);
 
 	/* End of menu creation */
 
 	gtk_widget_set_events (html_widget, GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
 
-	gtk_signal_connect (GTK_OBJECT (html), "title_changed",
-			    GTK_SIGNAL_FUNC (title_changed_cb), (gpointer)app);
-	gtk_signal_connect (GTK_OBJECT (html), "url_requested",
-			    GTK_SIGNAL_FUNC (url_requested), (gpointer)app);
-	gtk_signal_connect (GTK_OBJECT (html), "load_done",
-			    GTK_SIGNAL_FUNC (load_done), (gpointer)app);
-	gtk_signal_connect (GTK_OBJECT (html), "on_url",
-			    GTK_SIGNAL_FUNC (on_url), (gpointer)app);
-	gtk_signal_connect (GTK_OBJECT (html), "set_base",
-			    GTK_SIGNAL_FUNC (on_set_base), (gpointer)app);
-	gtk_signal_connect (GTK_OBJECT (html), "button_press_event",
-			    GTK_SIGNAL_FUNC (on_button_press_event), popup_menu);
-	gtk_signal_connect (GTK_OBJECT (html), "link_clicked",
-			    GTK_SIGNAL_FUNC (on_link_clicked), NULL);
-	gtk_signal_connect (GTK_OBJECT (html), "redirect",
-			    GTK_SIGNAL_FUNC (on_redirect), NULL);
-	gtk_signal_connect (GTK_OBJECT (html), "submit",
-			    GTK_SIGNAL_FUNC (on_submit), NULL);
-	gtk_signal_connect (GTK_OBJECT (html), "object_requested",
-			    GTK_SIGNAL_FUNC (object_requested_cmd), NULL);
+	g_signal_connect (html, "title_changed", G_CALLBACK (title_changed_cb), (gpointer)app);
+	g_signal_connect (html, "url_requested", G_CALLBACK (url_requested), (gpointer)app);
+	g_signal_connect (html, "load_done", G_CALLBACK (load_done), (gpointer)app);
+	g_signal_connect (html, "on_url", G_CALLBACK (on_url), (gpointer)app);
+	g_signal_connect (html, "set_base", G_CALLBACK (on_set_base), (gpointer)app);
+	g_signal_connect (html, "button_press_event", G_CALLBACK (on_button_press_event), popup_menu);
+	g_signal_connect (html, "link_clicked", G_CALLBACK (on_link_clicked), NULL);
+	g_signal_connect (html, "redirect", G_CALLBACK (on_redirect), NULL);
+	g_signal_connect (html, "submit", G_CALLBACK (on_submit), NULL);
+	g_signal_connect (html, "object_requested", G_CALLBACK (object_requested_cmd), NULL);
 
 #if 0
 	gtk_box_pack_start_defaults (GTK_BOX (hbox), GTK_WIDGET (html));
@@ -1156,13 +1073,8 @@ main (gint argc, gchar *argv[])
 
 	gtk_widget_show_all (app);
 
-	{
-		const char **args;
-		args = poptGetArgs(ctx);
-
-		if (args && args[0])
-			goto_url (args[0], 0);
-	}
+	if (argc > 1 && *argv [argc - 1] != '-')
+		goto_url (argv [argc - 1], 0);
 
 	gtk_main ();
 
